#include <Adafruit_MotorShield.h>

Adafruit_MotorShield AFMS = Adafruit_MotorShield();
Adafruit_DCMotor *myMotor1 = AFMS.getMotor(1);    // Rear Left
Adafruit_DCMotor *myMotor2 = AFMS.getMotor(2);    // Rear Right
Adafruit_DCMotor *myMotor3 = AFMS.getMotor(3);    // Front Right
Adafruit_DCMotor *myMotor4 = AFMS.getMotor(4);    // Front Left

String inputString = "";         // a string to hold incoming data
String bufString = "";

void setup() {
  pinMode(LED_BUILTIN, OUTPUT);

  digitalWrite(LED_BUILTIN, LOW);

  Serial.begin(19200);
  inputString.reserve(100);
  bufString.reserve(10);

  if (!AFMS.begin()) {         // create with the default frequency 1.6KHz
  // if (!AFMS.begin(1000)) {  // OR with a different frequency, say 1KHz
    Serial.println("Could not find Motor Shield. Check wiring.");
    while (1);
  }
  Serial.println("Motor Shield found.");

  // Set the speed to start, from 0 (off) to 255 (max speed)
  myMotor1->setSpeed(128);
  myMotor2->setSpeed(128);
  myMotor3->setSpeed(128);
  myMotor4->setSpeed(128);

  delay(300);
  Serial1.begin(19200);
}

void loop() {
  if (Serial1.available() > 0) {
    char inChar = Serial1.read();
    inputString += inChar;
    if (inChar == '\n') {
      cmd_get();
      inputString = "";
      bufString = "";
    }
  }

  delay(10);
}

void cmd_get(void) {
  int dataindex;
  char **e;
  String chkstr = inputString.substring( 0, 6 );

  if( chkstr == "CONECT" || chkstr == "DISCON" ) {
    Serial.println( "return" );
    return;
  }
  
  dataindex = inputString.indexOf(':') + 1;
  bufString = inputString.substring(dataindex, (dataindex + 2));
  const char *s = bufString.c_str();
  int cmd = (int)strtol(s, e, 16);

  Serial.print( "rcv data:" );
  Serial.print(inputString);
  Serial.print( "sub data:" );
  Serial.println( inputString.substring( 0, 6 ) );
  Serial.print( "cmd:" );
  Serial.println(cmd);

  switch( cmd ) {
    case 0:
      stp();
      break;
    case 1:
      forward();
      break;
    case 2:
      backward();
      break;
    case 3:
      right();
      break;
    case 4:
      left();
      break;
    case 5:
      forward_right();
      break;
    case 6:
      forward_left();
      break;
    case 7:
      backward_right();
      break;
    case 8:
      backward_left();
      break;
    case 9:
      cw();
      break;
    case 10:
      ccw();
      break;
    default:
      stp();
  }
}

void forward(void) {
    myMotor1->run(FORWARD);    // Rear Left
    myMotor2->run(FORWARD);    // Rear Right
    myMotor3->run(FORWARD);    // Front Right
    myMotor4->run(FORWARD);    // Front Left
}

void backward(void) {
    myMotor1->run(BACKWARD);    // Rear Left
    myMotor2->run(BACKWARD);    // Rear Right
    myMotor3->run(BACKWARD);    // Front Right
    myMotor4->run(BACKWARD);    // Front Left
}

void right(void) {
    myMotor1->run(BACKWARD);    // Rear Left
    myMotor2->run(FORWARD);    // Rear Right
    myMotor3->run(BACKWARD);    // Front Right
    myMotor4->run(FORWARD);    // Front Left
}

void left(void) {
    myMotor1->run(FORWARD);    // Rear Left
    myMotor2->run(BACKWARD);    // Rear Right
    myMotor3->run(FORWARD);    // Front Right
    myMotor4->run(BACKWARD);    // Front Left
}

void forward_right(void) {
    myMotor1->run(RELEASE);    // Rear Left
    myMotor2->run(FORWARD);    // Rear Right
    myMotor3->run(RELEASE);    // Front Right
    myMotor4->run(FORWARD);    // Front Left
}

void forward_left(void) {
    myMotor1->run(FORWARD);    // Rear Left
    myMotor2->run(RELEASE);    // Rear Right
    myMotor3->run(FORWARD);    // Front Right
    myMotor4->run(RELEASE);    // Front Left
}

void backward_right(void) {
    myMotor1->run(BACKWARD);    // Rear Left
    myMotor2->run(RELEASE);    // Rear Right
    myMotor3->run(BACKWARD);    // Front Right
    myMotor4->run(RELEASE);    // Front Left
}

void backward_left(void) {
    myMotor1->run(RELEASE);    // Rear Left
    myMotor2->run(BACKWARD);    // Rear Right
    myMotor3->run(RELEASE);    // Front Right
    myMotor4->run(BACKWARD);    // Front Left
}

void cw(void) {
    myMotor1->run(FORWARD);    // Rear Left
    myMotor2->run(BACKWARD);    // Rear Right
    myMotor3->run(BACKWARD);    // Front Right
    myMotor4->run(FORWARD);    // Front Left
}

void ccw(void) {
    myMotor1->run(BACKWARD);    // Rear Left
    myMotor2->run(FORWARD);    // Rear Right
    myMotor3->run(FORWARD);    // Front Right
    myMotor4->run(BACKWARD);    // Front Left
}

void stp(void) {
    myMotor1->run(RELEASE);    // Rear Left
    myMotor2->run(RELEASE);    // Rear Right
    myMotor3->run(RELEASE);    // Front Right
    myMotor4->run(RELEASE);    // Front Left
}
